package main

import(
	"fmt"
	"os"
	"io/ioutil"
	"path"
	"path/filepath"
	"eLD/signature"
	"eLD/config"
	"eLD/score"
	"io"
	"strings"
	"eLD/levenshtein"
)



func main() {  
	menu()

	//fmt.Println(levenshtein.LD([]rune("hallo ein bier"), []rune("hallo mein bier")))
}


//all files from a directory are parsed (not recursively) and the
//regular LD is built; slow! 
func processDirForRegLD(dir string){
	files, err := ioutil.ReadDir(dir)
	if err != nil {
	    fmt.Fprintf(os.Stderr, "error: %v\n", err)
        os.Exit(1)
	}


	var filesContent []string
	var fileNames []string

	for _, f := range files {
	    filename := path.Join(dir, f.Name())
	    fileExtension := filepath.Ext(filename)
		if fileExtension != ".txt" {
	    	continue	//jump to next file if not a txt file
	    }

	    fileNames = append(fileNames, filename)

	    // Read the entire file and print to the standard output
	    file, _ := os.Open(filename)
		b, _ := io.ReadAll(file)
		
		file.Close()

		tmp := strings.TrimRight(string(b), "\n")
		filesContent = append(filesContent, tmp)

		//_ = filesContent

		//printSlice(filesContent)
	}

	for i:=0; i<len(filesContent); i++ {
		for j:=i+1; j<len(filesContent); j++{		//change to j:=0 for complete all to all
			//fmt.Println(fileNames[j][0:32])
			//if (!strings.HasPrefix(fileNames[i], fileNames[j][0:37])) || (fileNames[i] == fileNames[j]) || (len(fileNames[i]) >= len(fileNames[j])){
			//	continue
			//}
			f1 := []rune(filesContent[i])
			f2 := []rune(filesContent[j])
			
			//fmt.Println(fileNames[i], " | ", fileNames[j])
			fmt.Println(fileNames[i], " | ", fileNames[j], " | ", levenshtein.LD(f1,f2))
		}
	}

	

}
//func printSlice(s []string) {
//	fmt.Printf("len=%d cap=%d %v\n", len(s), cap(s), s)
//}

func compareTwoFiles(file1 string, file2 string){
	fileExtension := filepath.Ext(file1)
	var sigA, sigB signature.Sig

	if fileExtension == ".txt" {
		sigA = signature.New(file1, config.C, config.N)		//C and N come from the config!
	}
	fileExtension = filepath.Ext(file2) 
	if fileExtension == ".txt" {
		sigB = signature.New(file2, config.C, config.N)		//C and N come from the config!
	} 

	res := score.New(&sigA, &sigB)
    res.Print(0.0)

}


//only makes the decision and forwards it
func processDirOrFile(dirOrFile string) {	
	if isDir(dirOrFile) {
		processDir(dirOrFile)
	} else {
		// file is not a directory
		processFile(dirOrFile)

	}
}

//all files from a directory are parsed (not recursively) and then 
//processFile is executed to check that it is actually a txt file
//and generates the signature
func processDir(dir string){
	files, err := ioutil.ReadDir(dir)
	if err != nil {
	    fmt.Fprintf(os.Stderr, "error: %v\n", err)
        os.Exit(1)
	}

	for _, f := range files {
	    processFile(path.Join(dir, f.Name()))
	}

}

//processFile checks that it is actually a txt file and generates signature
func processFile(filename string){
	fileExtension := filepath.Ext(filename)
	if fileExtension == ".txt" {
		sig := signature.New(filename, config.C, config.N)		//C and N come from the config!
		sig.Print()
	}
}


//compares two csv files again each other
func compare_two_csv(csv1path string, csv2path string, T float64) {
	csv1 := fileToStringArray(csv1path)
	csv2 := fileToStringArray(csv2path)

	i, j := 0, 0
	for i = 0; i<len(csv1); i++ {
		for j = 0; j<len(csv2); j++ {
			//print(i, " ", j, "--")
    		printScoreForSigs(csv1[i], csv2[j], T)
		}
	}
}

//performs an all-against-all comparison of one csv ignore duplicates
func compare_one_csv(csvPath string, T float64) {
	csv := fileToStringArray(csvPath)

	for i := 0; i<len(csv); i++ {
		for j:=i+1; j<len(csv); j++ {		//change to j:=0 for complete all to all
			//print(i, " ", j, "--")
			printScoreForSigs(csv[i], csv[j], T)
		}
	}
}

func printScoreForSigs(sigAs string, sigBs string, T float64) {
	sigA := signature.CreateSignatureFromString(sigAs)
    sigB := signature.CreateSignatureFromString(sigBs)
    //if (!strings.HasPrefix(sigA.Filename, sigB.Filename[0:37]) || (sigA.Filename == sigB.Filename) || (len(sigA.Filename) >= len(sigB.Filename))){
	//			print()
	//		} else {

    res := score.New(sigA, sigB)
    res.Print(T)
	//}
}

