package score

import(
	"fmt"
	"math"
	"eLD/signature"
	"eLD/config"
	"eLD/levenshtein"
	"os"
)

// for simplicity storing the complete signature;  could only store the filenames 
// digLD is needed for the significance
type Score struct{
	sigA *signature.Sig
	sigB *signature.Sig
	digLD int 				//real LD between sigA and sigB; stored for efficientcy to not recalc it
	eLD int 				//the estimated LD between the signatures 
	sigScore float64     	//significance score (delta)
}

//fake constructor to generate a sore given two signatures
func New(sigA *signature.Sig, sigB *signature.Sig) Score {
	if sigA.C != sigB.C || sigA.N != sigB.N{
		fmt.Fprintln(os.Stderr, "error: C or N of signatures not matching")
		os.Exit(1)
	}

	eLD, digLD := compare(sigA, sigB)

	score := Score{
		sigA: sigA,
		sigB: sigB,
		eLD: eLD,
		digLD: digLD,
		sigScore: -1,
	}

	score.significance()

	return score
}


//compares to signatures A and B
func compare(sigA *signature.Sig, sigB *signature.Sig) (int, int) {
	digDiff 		:= Abs(sigA.DigLen - sigB.DigLen)

	effectiveC 		:= float64(sigA.FileLen + sigB.FileLen) / float64(sigA.DigLen + sigB.DigLen)
	
	digLD      		:=  levenshtein.LD(sigA.Dig, sigB.Dig)
 
    scaledDigLD     :=  int(math.Round(float64( digLD - digDiff ) * effectiveC / (1.0 + config.EXPECTED_OVERLAP_TEXT )))
        
    fileLengthDiff  :=  Abs(sigA.FileLen - sigB.FileLen)

    return scaledDigLD + fileLengthDiff, digLD

}

//in order to decide if a match is relevant (i.e., are/are not considered related), we
//introduce the concept of the significance score which is pretty much a threshold when a
//result is printed or not
func (score *Score) significance_Peter() {
	//easier access in the follwoing
	sigA := score.sigA
	sigB := score.sigB

	shortSig, longSig 	:= intSort(sigA.DigLen, sigB.DigLen)

	DigDiff		:= longSig - shortSig
	expectSigLD := int(float64(shortSig) * (1.0-config.EXPECTED_OVERLAP_SIG)) + DigDiff

	actualLD 	:= score.digLD


	score.sigScore = float64(Abs(expectSigLD-actualLD)) / float64(expectSigLD)
	
	
	//score.sigScore = (longSig - ld) / shortSig
}

//in order to decide if a match is relevant (i.e., are/are not considered related), we
//introduce the concept of the significance score which is pretty much a threshold when a
//result is printed or not
func (score *Score) significance() {
	//easier access in the follwoing
	sigA := score.sigA
	sigB := score.sigB

	shortSig 	:= float64(min(sigA.DigLen, sigB.DigLen))
	longSig		:= float64(max(sigA.DigLen, sigB.DigLen))

	ld 			:= float64(score.digLD)
	
	score.sigScore = (longSig - ld) / shortSig
}


//Printing...
func (s Score) Print2(T float64) {
	shorter, longer := intSort(s.sigA.FileLen, s.sigB.FileLen)
	if (s.sigScore >= T) && (shorter*15 > longer) {
		fmt.Println(s.sigA.Filename, " | ", s.sigB.Filename, " | ", s.eLD, "|", s.sigScore)	
	
	}
	
	//fmt.Println(s.sigA.Filename, " | ", s.sigB.Filename, " | ", s.eLD)
	//fmt.Println(s.sigA.Filename, " | ", s.sigA.FileLen, " | ", s.sigB.Filename, " | ", s.sigB.FileLen, " | ", s.eLD)
}

//Printing...
func (s Score) Print(T float64) {
	shorter, longer := intSort(s.sigA.FileLen, s.sigB.FileLen)
	//fmt.Println(shorter, longer)
	if (shorter*10 < longer) {
		fmt.Println(s.sigA.Filename, " | ", s.sigB.Filename, " | ", s.eLD, " | ", 0, " | *", )	
	} else {
		fmt.Println(s.sigA.Filename, " | ", s.sigB.Filename, " | ", s.eLD, " | ", s.sigScore)	
	}
	
	//fmt.Println(s.sigA.Filename, " | ", s.sigB.Filename, " | ", s.eLD)
	//fmt.Println(s.sigA.Filename, " | ", s.sigA.FileLen, " | ", s.sigB.Filename, " | ", s.sigB.FileLen, " | ", s.eLD)
}



//returns the absolute value of x.
func Abs(x int) int {
	if x < 0 {
		return -x
	}
	return x
}


//given two integers, it returns the smaller
func min(a, b int) int {
    if a < b {
        return a
    }
    return b
}


//given two integers, it returns the larger
func max(a, b int) int {
    if a > b {
        return a
    }
    return b
}


func intSort(a,b int) (int, int) {
	if a > b {
        return b, a
    }
    return a, b
}