package signature

import (
	"fmt"
	"io/ioutil"
	"os"
	"bytes"
	"strings"
	"strconv"
	"eLD/config"
	//"hash/adler32"
	//"hash/fnv"
	"crypto/md5"
//	"encoding/hex"
	"encoding/binary"
	//"math/big"
)


type Sig struct {
	Filename string
	FileLen int
	C int
	N int
	DigLen int
	Dig []rune 
}

func New(filename string, C int, N int) Sig {
	//create an almost empty signature that will be filled along the way
	sig := Sig{filename, 0, C, N, 0, nil}
	parse_file_to_signature(&sig)
	return sig

}

//
func parse_file_to_signature(sig *Sig) {
	b, err := ioutil.ReadFile(sig.Filename)
	if err != nil {
        fmt.Fprintf(os.Stderr, "error: %v\n", err)
        os.Exit(1)
    }
    text := string(b) // convert content to a 'string'
   
    // fills in FileLen which equals string length
    sig.FileLen = len(text)
    

    if sig.FileLen < sig.N {
    	fmt.Fprintf(os.Stderr, "error: N too large; file not large enough")
        os.Exit(1)
    }

    //sig.Dig = compress_text(text, sig.C, sig.N)  //contains alterantive hashing algorithms
    sig.Dig = compress_text_rabinKarb(text, sig.C, sig.N)
    sig.DigLen = len(sig.Dig)

    //fmt.Println(str) // print the content as a 'string'


}


//function was only used for testing the runtime of other implementations
//for compressing the compress_text_rabinKarb is used
func compress_text(text string, C int, N int) []rune {
	var end int  = len(string(text)) - N
	var digBuffer bytes.Buffer


	//fnvH := fnv.New32a()
	//bi := big.NewInt(0)
	var h uint = 1

	for p := 0; p<end; p++ {

		//djb2 hash
		//var h uint = 5381
        //for _, c := range text[p:p+N] {
        //        h = (((h << 5) + h) + uint(c)) &0xFFFF
		//}
		
		//fnv
        //fnvH.Write([]byte(text[p:p+N-1]))
        //h := uint(fnvH.Sum32())

        
        //MD5
        h1 := md5.New()
		h1.Write([]byte(text[p:p+N-1]))
		tmp := h1.Sum(nil)[0:4]
 		h = uint(binary.BigEndian.Uint32(tmp))
		//fmt.Println("ha %x", h)
		//hexstr := hex.EncodeToString(h1.Sum(nil))
		//bi.SetString(hexstr, 16)
		//h := uint(myAtoi(bi.String()))


        
		//see if it matches signature
		if h % uint(C) == 0 {
			digBuffer.WriteString(string(config.ALPHABET[h % uint(len(config.ALPHABET))]))
		}
	}
	
	//fmt.Println(digBuffer.String())
	return []rune(digBuffer.String())
}	



//https://www.golangprograms.com/golang-program-for-implementation-of-rabin-karp.html
func compress_text_rabinKarb(text string, C int, N int) []rune {
	var end  = len(string(text)) - N
	var digBuffer bytes.Buffer
	
	//parameters needed for Rabin Karb
	const base = 16777619     // a prime
    
    //The value of x would be "pow(d, N-1) % q"
    var mult uint32 = 1 // mult = base^(m-1)
	for i := 0; i < N-1; i++ {
		mult = (mult * base)
	}

    // Calculate the hash value of first window of text
    var h uint32
	for i := 0; i < int(N); i++ {
		h = (h*base + uint32(text[i]))
	}    
	
	//slide over the text character by character	
	for p := 1; p<end+1; p++ {
		
		//design decision to first check if feature matches
		if h % uint32(C) == 0 {
			//do not use string as it becomes incredibly slow 
			digBuffer.WriteString(string(config.ALPHABET[h % uint32(len(config.ALPHABET))]))
		}	

		//update hash be removing p-1 and adding p+N-1
		h = h - mult*uint32(text[p-1])
		h = h*base + uint32(text[p+N-1])
	}		
	
	return []rune(digBuffer.String())
}	

//prints a signature separated by space
func (sig *Sig) Print() {
	fmt.Print(sig.Filename+",") 
	fmt.Print(sig.FileLen)
	fmt.Print(",")
	fmt.Print(sig.C)
	fmt.Print(",")
	fmt.Print(sig.N)
	fmt.Print(",")
	fmt.Print(sig.DigLen)
	fmt.Print(",")
	fmt.Println(string(sig.Dig))
}




func CreateSignatureFromString(sigAsString string) *Sig {
	s := strings.Split(sigAsString, ",")
	sig := Sig {
		Filename: 	s[0],
		FileLen: 	myAtoi(s[1]),
		C: 			myAtoi(s[2]),
		N: 			myAtoi(s[3]),
		DigLen:  	myAtoi(s[4]),
		Dig: 		[]rune(s[5]),
	}
	return &sig
}


//ignores the error and only returns the int - for simplicity
func myAtoi(s string) int {
	t, _ := strconv.Atoi(s)
	return t
}



